#!/usr/bin/env python2

import os,sys,re


def natural_sort(l):
    convert = lambda text: int(text) if text.isdigit() else text.lower() 
    alphanum_key = lambda key: [ convert(c) for c in re.split('([0-9]+)', key) ] 
    return sorted(l, key = alphanum_key)


def is_number(s):
    try:
        float(s)
        return True
    except ValueError:
        return False

def main():
    # useage: python scripts/comparing_outgroup_patient.py --out outgroup_ctl.txt --patient patient_ctl.txt
    ctl_out=sys.argv[sys.argv.index("--out")+1]
    ctrl_out=open(ctl_out)
    out_dic={} #dictionary with the name of the output directory for the fastq as the key, and containing another dictionary with the path to the reference("ref"), and the path to the fasta 
    dep_varcall=""
    dep_bed=""
    dep_regions=""
    for line in ctrl_out:
        spl=line.replace("\n","").replace("\r","").split("\t")
        name=spl[0]
        ref=spl[1]
        fasta=spl[2]
        out_dic[name]={"ref":ref,"fasta":fasta}


    
    ctl_patient=sys.argv[sys.argv.index("--patient")+1]
    ctrl_patient=open(ctl_patient)
    patient_dic={} #dictionary with the name of the output directory for the fastq as the key, and containing another dictionary with the path to the reference("ref") and the path to the fastq files ("R1","R2")
    dep_varcall=""
    dep_bed=""
    dep_regions=""
    for line in ctrl_patient:
        spl=line.replace("\n","").replace("\r","").split("\t")
        name=spl[0]
        ref=spl[1]
        R1=spl[2]
        R2=spl[3]
        patient_dic[name]={"ref":ref,"R1":R1,"R2":R2}

    
    dic_mut_patient={}
    for name in patient_dic:
        f=open(name+"/snps.filt.vcf")
        header=""
        for lin in f:
            #print lin
            if lin[0]!="#":
                spl=lin.replace("\n","").replace("\r","").split("\t")
                if len(spl[3])==1 and len(spl[4])==1: #here we look at SNVs
                    mut=spl[1]+"_"+spl[3]+"_"+spl[4] #we create and identifier for each mutations in the form: position_old_new
                    if mut not in dic_mut_patient:
                        dic_mut_patient[mut]=[]
                    dic_mut_patient[mut].append(lin)
                    #print "dic", fil, mut, lin
                    
                elif len(spl[3])==len(spl[4]): #here we look at a complex SNVs
                    for i in range(0,len(spl[3])):
                        if spl[3][i]!=spl[4][i]: #we compare the old vs the new and wee look for the positions that missmatch, and we ad tham to dic as SNVs
                            mut=str(int(spl[1])+i)+"_"+spl[3][i]+"_"+spl[4][i]#we create and identifier for each mutations in the form: position_old_new
                            if mut not in dic_mut_patient:
                                dic_mut_patient[mut]=[]
                            dic_mut_patient[mut].append(lin)
                            
                else: # here we look at an indel
                    mut=spl[1]+"_"+spl[3]+"_"+spl[4]
                    if mut not in dic_mut_patient:
                        dic_mut_patient[mut]=[]
                    dic_mut_patient[mut].append(lin)



            else:
                header+=lin

    #Here we create a dictionary with for each position in the reference genome a count of how many of the outgroups isolates have pased snippys treshlods at this position.
    dic_BH_coord={} 
    for name in out_dic:
        f=open(name+"/snps.aligned.fa")
        seq=""
        for lin in f:
            if lin[0]!=">":
                seq+=lin[:-1]
        for i in range(0,len(seq)):
            if str(i) not in dic_BH_coord:
                dic_BH_coord[str(i)]=0
            if seq[i]!="-" and seq[i]!="N": 
                dic_BH_coord[str(i)]+=1

    
    dic_BH={}
    for name in out_dic:
        f=open(name+"/snps.filt.vcf")
        header=""
        for lin in f:
            if lin[0]!="#":
                spl=lin[:-1].split("\t")
                
                if len(spl[3])==1 and len(spl[4])==1: #here we look at SNVs
                    mut=spl[1]
                    if mut not in dic_BH:
                        dic_BH[mut]=[]
                    dic_BH[mut].append(lin)
                    
                elif len(spl[3])==len(spl[4]): #here we look at a complex SNVs
                    for i in range(0,len(spl[3])):
                        if spl[3][i]!=spl[4][i]:
                            mut=str(int(spl[1])+i)
                            if mut not in dic_BH:
                                dic_BH[mut]=[]
                            dic_BH[mut].append(lin)
                            
                else: # here we look at an indel
                    mut=spl[1]
                    if mut not in dic_BH:
                        dic_BH[mut]=[]
                    dic_BH[mut].append(lin)

            else:
                header+=lin


    new=open("to_apply.vcf","w")
    new.write("##fileformat=VCFv4.2\n#CHROM	POS	ID	REF	ALT	QUAL	FILTER	INFO	FORMAT	ancestor\n")
    already=[]
    for mut in natural_sort(dic_mut_patient): #we look at the mutations position by position
        tot=0 #initiation of tot
        tot_rev=0
        if dic_BH_coord[mut.split("_")[0]]!=0: #The region is covered in the outgroups
            if mut.split("_")[0] in dic_BH:
                tot=len(dic_BH[mut.split("_")[0]]) #number of ougroup isolates that are covered in this position
            count_in_patient=float(len(dic_mut_patient[mut]))/len(patient_dic) #number of patient isolate the mutation is present in
            count_in_outgroup=float(tot)/dic_BH_coord[mut.split("_")[0]] #number of outgroup isolate the mutation is present in
            if count_in_patient >0.5 and count_in_outgroup>0.5 and dic_mut_patient[mut][0] not in already: #here the mutation is present in the majority of the isolates and in the majority of the outgroup
                new.write(dic_mut_patient[mut][0])
                already.append(dic_mut_patient[mut][0])
                
            else:#here the mutation is not present in the majority of the isolates and in the majority of the outgroup we don't keep the mutation
                pass

        else: #the region is not covered in the outgroups
            count_in_patient=float(len(dic_mut_patient[mut]))/len(patient_dic)
            if count_in_patient>0.5 and dic_mut_patient[mut][0] not in already: #the mutation is in the majority of the patient isolate. we keep it
                new.write(dic_mut_patient[mut][0])
                already.append(dic_mut_patient[mut][0])

            else:#the mutation is not present in the majority of isolates. we don't keep it.
                pass

    new.close()


    #bgzipping and in dexing the output vcf
    cmd='bgzip -c to_apply.vcf > to_apply.vcf.gz'
    print cmd
    os.system(cmd)

    cmd='tabix to_apply.vcf.gz'
    print cmd
    os.system(cmd)

    
    #We finally apply the variants on the vcf to the refrence sequence
    cmd='cat '+ref+' | vcf-consensus to_apply.vcf.gz > recons_ancestor_snippy.fa'
    print cmd
    os.system(cmd)


main()
