library(ape)
library(tidyverse)
library(forcats)
library(parallel)
library(foreach)
library(doParallel)
library(doMC)

registerDoMC(cores=future::availableCores())

#Input files
############



#gff file 
gff = read.gff("recons_ancestor_snippy_noSeq.txt")
gff = gff[gff$type=="CDS",]

#Mutation counts for each of the hypermutator groups
mutations_patient_dnaQ<-read.csv("mutation_counts_Patient_dnaQ_forSim.txt",sep="\t",header=T,stringsAsFactors = T)
mutations_patient_mutYM<-read.csv("mutation_counts_Patient_mutYM_forSim.txt",sep="\t",header=T,stringsAsFactors = T)
mutations_patient_WT<-read.csv("mutation_counts_Patient_WT_forSim.txt",sep="\t",header=T,stringsAsFactors = T)


#Input values
#############

#genome length
genome_length=4899042

#Numbers of SNV to be redistributed for each groups
tot_mut_patient_dnaQ=3864
tot_mut_patient_mutYM=2316
tot_mut_patient_WT=71
tot_mut_patient_merged=tot_mut_patient_dnaQ+tot_mut_patient_mutYM+tot_mut_patient_WT


#Creating a vector containing a list of gene with a number of repetition equal to the gene length

#Creating the vector
gene_order=c()
gene_list=c(rep("inter",4899042))
length=c()
for (i in 1:dim(gff)[1]) {
  s=gff$start[i]
  e=gff$end[i]
  name=str_match(gff$attributes[i],".*Name=(.*?);.*")[2]
  
  if (is.na(name)){
    #If no names for the gene we use the ID instead
    name=str_match(gff$attributes[i],".*ID=(.*?);.*")[2]
  }
  
  length=c(length,c(e-s+1))
  gene_order=c(gene_order,name)
  for (j in s:e){
    
    if (gene_list[j]=="inter"){
      gene_list[j]=name
    }
    else{
    gene_list[j]=paste(gene_list[j],name)
    }
  }
}

gene_length=data.frame(gene=gene_order,length)
gene_order=sort(gene_order)


#the simulation function
########################

#takes in the number of cycles and the gene list and the number of mutations that needs to be redistributed
simulation<-function(cycles,gene_list,mutations){
  
  #we run in parallel each cycles
  results_sim<-foreach (i=1:cycles, .combine=cbind ,.multicombine = TRUE) %dopar% {
    random_numbers<- sample(1:4899035, mutations) #we pick a random position for each mutation that needs to be redistributed
    sim_in_gene=gene_list[random_numbers] #Match the random positions to the list of gene to identify which genes were mutated
    
    for (j in 1:length(sim_in_gene)){
      geneatpos=sim_in_gene[j]
      nbgene=str_count(geneatpos," ") #we count the number of genes that cover this position
      if(nbgene==1){ #more than one gene. we pick one randomly
        g=unlist(strsplit(geneatpos," "))
        sim_in_gene[j]=sample(g, 1)
      }
    }
    as.data.frame(table(factor(sim_in_gene,levels=gene_order)))[,2] #we creat a table with the mutation counts for each genes
    }
  results_sim
}


cyc=100000

#The simulations are separated in 10 groupes and then concatenated as this speedsup the process

#Simulation for WT isolates
start<-Sys.time()
count_WT1<-simulation(cyc/10,gene_list,tot_mut_patient_WT)
count_WT2<-simulation(cyc/10,gene_list,tot_mut_patient_WT)
count_WT3<-simulation(cyc/10,gene_list,tot_mut_patient_WT)
count_WT4<-simulation(cyc/10,gene_list,tot_mut_patient_WT)
count_WT5<-simulation(cyc/10,gene_list,tot_mut_patient_WT)
count_WT6<-simulation(cyc/10,gene_list,tot_mut_patient_WT)
count_WT7<-simulation(cyc/10,gene_list,tot_mut_patient_WT)
count_WT8<-simulation(cyc/10,gene_list,tot_mut_patient_WT)
count_WT9<-simulation(cyc/10,gene_list,tot_mut_patient_WT)
count_WT10<-simulation(cyc/10,gene_list,tot_mut_patient_WT)

count_WT=as.data.frame(gene_order)
count_WT<-cbind(count_WT,count_WT1,count_WT2,count_WT3,count_WT4,count_WT5,
                  count_WT6,count_WT7,count_WT8,count_WT9,count_WT10)
end<-Sys.time()
print(end-start)



#Simulation for dnaQ isolates
start<-Sys.time()
count_dnaQ1<-simulation(cyc/10,gene_list,tot_mut_patient_dnaQ)
count_dnaQ2<-simulation(cyc/10,gene_list,tot_mut_patient_dnaQ)
count_dnaQ3<-simulation(cyc/10,gene_list,tot_mut_patient_dnaQ)
count_dnaQ4<-simulation(cyc/10,gene_list,tot_mut_patient_dnaQ)
count_dnaQ5<-simulation(cyc/10,gene_list,tot_mut_patient_dnaQ)
count_dnaQ6<-simulation(cyc/10,gene_list,tot_mut_patient_dnaQ)
count_dnaQ7<-simulation(cyc/10,gene_list,tot_mut_patient_dnaQ)
count_dnaQ8<-simulation(cyc/10,gene_list,tot_mut_patient_dnaQ)
count_dnaQ9<-simulation(cyc/10,gene_list,tot_mut_patient_dnaQ)
count_dnaQ10<-simulation(cyc/10,gene_list,tot_mut_patient_dnaQ)

count_dnaQ=as.data.frame(gene_order)
count_dnaQ<-cbind(count_dnaQ,count_dnaQ1,count_dnaQ2,count_dnaQ3,count_dnaQ4,count_dnaQ5,
                  count_dnaQ6,count_dnaQ7,count_dnaQ8,count_dnaQ9,count_dnaQ10)
end<-Sys.time()
print(end-start)


#Simulation for dnaQ + mutYM isolates
start<-Sys.time()
count_mutYM1<-simulation(cyc/10,gene_list,tot_mut_patient_mutYM)
count_mutYM2<-simulation(cyc/10,gene_list,tot_mut_patient_mutYM)
count_mutYM3<-simulation(cyc/10,gene_list,tot_mut_patient_mutYM)
count_mutYM4<-simulation(cyc/10,gene_list,tot_mut_patient_mutYM)
count_mutYM5<-simulation(cyc/10,gene_list,tot_mut_patient_mutYM)
count_mutYM6<-simulation(cyc/10,gene_list,tot_mut_patient_mutYM)
count_mutYM7<-simulation(cyc/10,gene_list,tot_mut_patient_mutYM)
count_mutYM8<-simulation(cyc/10,gene_list,tot_mut_patient_mutYM)
count_mutYM9<-simulation(cyc/10,gene_list,tot_mut_patient_mutYM)
count_mutYM10<-simulation(cyc/10,gene_list,tot_mut_patient_mutYM)

count_mutYM=as.data.frame(gene_order)
count_mutYM<-cbind(count_mutYM,count_mutYM1,count_mutYM2,count_mutYM3,count_mutYM4,count_mutYM5,
                  count_mutYM6,count_mutYM7,count_mutYM8,count_mutYM9,count_mutYM10)
end<-Sys.time()
print(end-start)



#Merging the data of the different groupes together
count_merged=cbind(gene_order,(count_dnaQ[,2:(cyc+1)]+count_mutYM[,2:(cyc+1)]+count_WT[,2:(cyc+1)]))




#creating output data table and stat calculation group by group
###############################################################

out=cbind(gene_length[order(gene_length$gene),])

#mutYM
out=cbind(out,
          SNV_mutYM=mutations_patient_mutYM[order(mutations_patient_mutYM$gene),][,2],
          Mean_mutYM=rowMeans(count_mutYM[,2:(cyc+1),]),
          SD_mutYM=apply(count_mutYM[,2:(cyc+1),], 1, sd))

#Calculation zscore
out=out %>% mutate(zscr_mutYM= (((SNV_mutYM) - Mean_mutYM)/SD_mutYM))


#Calculation of the empirical pvalue
Empirical_pvalue_mutYM<-foreach (i=1:length(gene_order), .combine=rbind ,.multicombine = TRUE) %dopar% {
  in_experiment=out$SNV_mutYM[i]
  1-ecdf(as.integer(count_mutYM[i,2:(cyc+1)]))(in_experiment-1)
}

out=cbind(out,EP_mutYM=Empirical_pvalue_mutYM)




#dnaQ
out=cbind(out,
          SNV_dnaQ=mutations_patient_dnaQ[order(mutations_patient_dnaQ$gene),][,2],
          Mean_dnaQ=rowMeans(count_dnaQ[,2:(cyc+1),]),
          SD_dnaQ=apply(count_dnaQ[,2:(cyc+1),], 1, sd))


#Calculation zscore
out=out %>% mutate(zscr_dnaQ= (((SNV_dnaQ) - Mean_dnaQ)/SD_dnaQ))

#Calculation of the empirical pvalue
Empirical_pvalue_dnaQ<-foreach (i=1:length(gene_order), .combine=rbind,.multicombine = TRUE ) %dopar% {
  in_experiment=out$SNV_dnaQ[i]
  1-ecdf(as.integer(count_dnaQ[i,2:(cyc+1)]))(in_experiment-1)
}
out=cbind(out,EP_dnaQ=Empirical_pvalue_dnaQ)



#WT
out=cbind(out,
          SNV_WT=mutations_patient_WT[order(mutations_patient_WT$gene),][,2],
          Mean_WT=rowMeans(count_WT[,2:(cyc+1),]),
          SD_WT=apply(count_WT[,2:(cyc+1),], 1, sd))


#Calculation zscore
out=out %>% mutate(zscr_WT= (((SNV_WT) - Mean_WT)/SD_WT))


#Calculation of the empirical pvalue
Empirical_pvalue_WT<-foreach (i=1:length(gene_order), .combine=rbind ,.multicombine = TRUE) %dopar% {
  in_experiment=out$SNV_WT[i]
  1-ecdf(as.integer(count_WT[i,2:(cyc+1)]))(in_experiment-1)
}
out=cbind(out,EP_WT=Empirical_pvalue_WT)







#merged
out=cbind(out,
          SNV_merged=mutations_patient_mutYM[order(mutations_patient_mutYM$gene),][,2]+mutations_patient_dnaQ[order(mutations_patient_dnaQ$gene),][,2]+mutations_patient_WT[order(mutations_patient_WT$gene),][,2],
          Mean_merged=rowMeans(count_merged[,2:(cyc+1),]),
          SD_merged=apply(count_merged[,2:(cyc+1),], 1, sd))



#Calculation zscore
out=out %>% mutate(zscr_merged= (((SNV_merged) - Mean_merged)/SD_merged))


#Calculation of the empirical pvalue
Empirical_pvalue_merged<-foreach (i=1:length(gene_order), .combine=rbind,.multicombine = TRUE ) %dopar% {
  in_experiment=out$SNV_merged[i]
  1-ecdf(as.integer(count_merged[i,2:(cyc+1)]))(in_experiment-1)
}
out=cbind(out,EP_merged=Empirical_pvalue_merged)



#writing output table
write.table(out, "Analysis_simulation_Patient_full_100000_cycle.tsv", append = FALSE, sep = "\t", quote = FALSE,dec = ".",row.names = F, col.names = TRUE)


