#!/usr/bin/env python2

import os, sys,re,random,time,numpy
from scipy.stats import norm
import scipy

def is_number(s):
    try:
        float(s)
        return True
    except ValueError:
        return False



def revers(s):
    #takes an DNA string S and return a reverted str
    dic={"A":"T","T":"A","C":"G","G":"C","a":"t","t":"a","c":"g","g":"c"}
    new_seq=""
    for i in s:
        new_seq=new_seq+dic[i]
    return new_seq[::-1]



def site_count(seq):
    #take a DNA sequence and return numbers of S and NS sites.
    degen_code= {'TAA':2,'TAG':2,'TGA':1,'GCA':4,
'GCC':4,'GCG':4,'GCT':4,'TGC':2,'TGT':2,
'GAC':2,'GAT':2,'GAA':2,'GAG':2,'TTC':2,
'TTT':2,'GGA':4,'GGC':4,'GGG':4,'GGT':4,
'CAC':2,'CAT':2,'ATA':3,'ATC':3,'ATT':3,
'AAA':2,'AAG':2,'CTA':4,'CTC':4,'CTG':4,
'CTT':4,'TTA':2,'TTG':2,'ATG':1,'AAC':2,
'AAT':2,'CCA':4,'CCC':4,'CCG':4,'CCT':4,
'CAA':2,'CAG':2,'AGA':2,'AGG':2,'CGA':4,
'CGC':4,'CGG':4,'CGT':4,'AGC':2,'AGT':2,
'TCA':4,'TCC':4,'TCG':4,'TCT':4,'ACA':4,
'ACC':4,'ACG':4,'ACT':4,'GTA':4,'GTC':4,
'GTG':4,'GTT':4,'TGG':1,'TAC':2,'TAT':2}

    if len(seq)%3==0:
        n=3
        codons=[seq[i:i+n] for i in range(0, len(seq), n)]
        S=0
        NS=0
        S_s=0
        NS_s=0
        test_s=0
        test_ns=0
        for codon in codons:
            S_s+=0/3+0/3+float(degen_code[codon]-1)/(4-1)
            test_s+=0/3+0/3+round(float(degen_code[codon]-1)/(4-1),1)
            NS_s+=3/3+3/3+(4-float(degen_code[codon]))/(4-1)
            test_ns+=3/3+3/3+round(float(degen_code[codon]-1)/(4-1),1)
        return{"S_s":S_s,"NS_s":NS_s}
    else:
        print ("ERROR: incomplete DNA seq (not multiple of 3)")
        return(0)


def main():
    
    start=time.time() #starting time

    
    #Isolates groups
    WT=["2B3","1G1","7B1","8B1"]
    mutYM=["1B1a","1B1b","3B1"]
    E9G_mut=["envZ",
"A_03098",
"A_02012",
"dnaQ",
"A_02072",
"A_04082",
"cheD",
"A_00560",
"A_03455",
"yhhQ",
"fliF",
"kefC_3",
"A_01313",
"gstB_2",
"ygaZ",
"A_03291",
"icd_1",
"ppsR",
"A_03713",
"lpxL_2",
"sucA",
"tdhA",
"ptsJ",
"gsiD_5"]




    #Recovering the gene info from the gff file
    #########################
                
    #initiating dictionary containing all the genes informations
    gene_info={}

    #initiating value to collect the full seuqnce of the genome
    seq=""

    #indice to indicate wether we are in the sequence part of the gff 
    s=0
    
    #We want to recover gene informations from a gff file:
    gff=open("recons_ancestor_snippy.gff")

    genic=0
    #we read the file line by line and we split the elements separated by a tab into a list
    for lin in gff:
        spl=lin[:-1].split("\t")
        if lin[0]!="#": #we pass the header of the gff
            
            if lin[0]==">": #we have passed the gene informations and are entring the sequence part of the gff
                s=1
                
            elif s!=1: #We are still in the gene information part of the gff
                
                if spl[2]=="CDS": #We only look at CDSs
                    #Locus ID and gene name are recivered using regular expressions
                    name=re.findall("Name=(.+?);",spl[8])
                    ID=re.findall("ID=(.+?);",spl[8])
                    #We extract the begining (b), end (e), orientation(c) and length of the gene
                    b=int(spl[3])-1
                    e=int(spl[4])
                    c=spl[6]
                    length=int(spl[4])-int(spl[3])
                    genic+=length
                    #here we identify the gene by its name or if not available it locus ID
                    if name:
                        #print name
                        gene_info[name[0]]=[length,b,e,c]
                    else:
                        #print ID
                        gene_info[ID[0]]=[length,b,e,c]
            else: #we are in the sequence part of the gff and we collect the sequence of the genome
                seq+=lin[:-1]




    for i in gene_info:
        if gene_info[i][3]=="+":
            seq_gene=seq[gene_info[i][1]:gene_info[i][2]]
            gene_info[i].append(seq_gene)
        else:
            seq_gene=revers(seq[gene_info[i][1]:gene_info[i][2]])
            gene_info[i].append(seq_gene)


    

    #Recovering the information from the patient
    ############################################
    
    #initiation of dictionary containing for each gene the number of non synonymous (NS) and synonymous (S) SNVs
    mut={"PT":{},"MA":{}}

    #dictionary of the mutational spectra for each gene
    mutspectra={}

    #extracting the mutations from the annotation file
    patient_fil=open("all_mut_snpSift.tsv")
    tot=0
    #we read the file line by line and we split the elements separated by a tab into a list
    for lin in patient_fil:
        spl=lin.replace("\n","").replace("\r","").split("\t")
        if len(spl[2])==1 and len(spl[3])==1: #we look only at non intergenic SNVs
            iso=spl[-1].split(",")
            tot+=1                                                                     
            if "INTERGENIC" not in spl[4]:
                gene_number=len(spl[8].split(","))
                for index in range(0,gene_number):
                    #here we identify the gene by its name or if not available it locus ID
                    name=spl[8].split(",")[index]
                    if name not in mut["PT"]:
                        mut["PT"][name]={"NS":0,"S":0}
                    #We initiate the mut dictionarries for the gene in question               
                    if "NON_SYNONYMOUS_CODING" in spl[4].split(",")[index] or 'STOP_GAINED' in spl[4].split(",")[index]:
                        mut["PT"][name]["NS"]+=1 
                    else:
                        mut["PT"][name]["S"]+=1
                        
    patient_fil.close()

    for i in gene_info:
         if i not in mut["PT"]:
             mut["PT"][i]={"NS":0,"S":0}






    #gathering mutation from mutation accumulation experiment.

    #extracting the mutations from the annotation file
    MA_fil=open("all_mut_snpSift_unique_MA.tsv")
    tot=0
    #we read the file line by line and we split the elements separated by a tab into a list
    for lin in MA_fil:
        spl=lin.replace("\n","").replace("\r","").split("\t")
        #print spl
        if len(spl[2])==1 and len(spl[3])==1: #we look only at non intergenic SNVs
            iso=spl[-1].split(",")
            tot+=1                                                                    
            if "INTERGENIC" not in spl[4]:
                #print spl
                gene_number=len(spl[8].split(","))
                if gene_number!=1:
                    #print spl
                    pass
                for index in range(0,gene_number):
                    #here we identify the gene by its name or if not available it locus ID
                    name=spl[8].split(",")[index]
                    if name not in mut["MA"]:
                        mut["MA"][name]={"NS":0,"S":0}
                        
                    if "NON_SYNONYMOUS_CODING" in spl[4].split(",")[index] in spl[4].split(",")[index]:
                        mut["MA"][name]["NS"]+=1 
                    elif 'STOP_GAINED' not in spl[4].split(",")[index]:
                        mut["MA"][name]["S"]+=1

    MA_fil.close()
    for i in gene_info:
         if i not in mut["MA"]:
             mut["MA"][i]={"NS":0,"S":0}


    new=open("mutation_counts_all_isolates_and_sites.txt","w")
    new.write("24 shared genes\tgene\tlength\tNS sites\tS sites\tPT_NS\tPT_S\tPT_dN\tPT_dS\tMA_NS\tMA_S\tMA_dN\tMA_dS\tPT_dNdS\tMA_dNdS\n")
    for i in gene_info:
        #print gene_info[i]
        sites=site_count(gene_info[i][-1][:-3])
        PT_dN=float(mut["PT"][i]["NS"])/sites["NS_s"]
        PT_dS=float(mut["PT"][i]["S"])/sites["S_s"]
        
        if PT_dS!=0:
            PT_dNdS_raw=PT_dN/PT_dS
            PT_dNdS=float('%.2g' % PT_dNdS_raw)

        else:
            PT_dNdS="NA"
            
        MA_dN=float(mut["MA"][i]["NS"])/sites["NS_s"]
        MA_dS=float(mut["MA"][i]["S"])/sites["S_s"]
        
        if MA_dS!=0:
            MA_dNdS_raw=MA_dN/MA_dS
            MA_dNdS=float('%.2g' % MA_dNdS_raw)
        else:
            MA_dNdS="NA"
        if i in E9G_mut:
            shared="1"
        else:
            shared="0"
        new.write(shared+"\t"+i+"\t"+str(len(gene_info[i][-1]))+"\t"+str(round(sites["NS_s"],2))+"\t"+str(round(sites["S_s"],2))+"\t"+str(mut["PT"][i]["NS"])+"\t"+str(mut["PT"][i]["S"])+"\t"+str(float('%.2g' % PT_dN))+"\t"+str(float('%.2g' % PT_dS))+"\t"+str(mut["MA"][i]["NS"])+"\t"+str(mut["MA"][i]["S"])+"\t"+str(float('%.2g' % MA_dN))+"\t"+str(float('%.2g' % MA_dS))+"\t"+str(PT_dNdS)+"\t"+str(MA_dNdS)+"\n")
    new.close()


    
main()

#26jan2021: created by modifying for_simulation_in_R.py
#01fev2021: added site count fuction to count number of NS and S sites
#11mar2021: added significant figures calculation
